//  MODULE:   SocketSetupAndListen.c
//
//  PURPOSE:	Waits on a socket connection and then creates a thread
//				to handle the request
//
//  FUNCTIONS:
//
//		DWORD WINAPI SocketSetupAndListen(LPVOID lpV)
//
//  COMMENTS:
//
//  AUTHOR: Shane Hebert
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <winsock.h>
#include "service.h"
#include "processinfo.h"
#include "mpiserve.h"
#include "SocketReadAndWrite.h"
#include "MPIServePacket.h"
#include "MPIJobQueue.h"
#include "debuglog.h"

//
//  FUNCTION: SocketSetupAndListen(LPVOID lpV)
//
//  PURPOSE: This is the actual thread that listens
//				for connections and fires off processes
//
//  PARAMETERS:
//		lpV	-	not used in function.  it is needed
//				to satisfy the prototype for a thread
//
//  RETURN VALUE:
//		status if the thread terminates because of an error
//
//  COMMENTS:
//
DWORD WINAPI SocketSetupAndListen(LPVOID lpV)
{
	LONG nStatus;
	WORD wPortNumber;
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	BOOL wsaInit=FALSE;
	SOCKET sListener;
	SOCKET sAccept;
	char szMsg[256];

	nStatus = GetPortNumber(&wPortNumber);
	sprintf(szMsg,"Started listening on port [%d]",wPortNumber);
	WriteDebugLog(szMsg);
	if (ERROR_SUCCESS != nStatus)
		goto cleanup;
	nStatus = WSAStartup(wVersionRequested,&wsaData);
	if (SOCKET_ERROR == nStatus)
		goto cleanup;
	wsaInit = TRUE;
	WriteDebugLog("WSA started fine");
	{
		MPIJobQueueInitialize();
		{
			SOCKADDR_IN aLocalAddress;

			sListener = socket(AF_INET,SOCK_STREAM,0);
			if (INVALID_SOCKET == sListener)
				goto cleanup;
			memset(&aLocalAddress,0,sizeof(SOCKADDR_IN));
			aLocalAddress.sin_port = htons(wPortNumber);
			aLocalAddress.sin_family = AF_INET;
			nStatus = bind(sListener,(PSOCKADDR)&aLocalAddress,sizeof(SOCKADDR_IN));
			if (SOCKET_ERROR == nStatus)
				goto cleanup;
		}
		WriteDebugLog("Listening for connections");
		nStatus = listen(sListener,5);
		if (SOCKET_ERROR == nStatus)
			goto cleanup;
		ReportInfoToMessageLog(TEXT("Service Started"));
		while (TRUE)
		{
			DWORD dwThreadID;
			HANDLE hThread;
			TASK_TYPE eTask;

			sAccept = accept(sListener,NULL,NULL);
			WriteDebugLog("Accepted a connection");
			if (INVALID_SOCKET == sAccept)
				goto cleanup;
			nStatus = RecvSocketN(sAccept,(BYTE*)&eTask,sizeof(TASK_TYPE));
			if (SOCKET_ERROR == nStatus)
			{
				nStatus = GetLastError();
				goto cleanup;
			}
			/*
				It is the duty of the thread to retrieve what information
				it needs from the socket passed to it, perform its job,
				handle all other communications with the connecting process,
				and then close the socket when finished.
			*/
			switch (eTask)
			{
			case MPI_SERVE_QUERY_INFO:
				WriteDebugLog("It is a request to Query Info");
				hThread = CreateThread(NULL,				/* NULL security attibutes */
										0,					/* default stack size */
										MPIServeQueryInfo,	/* thread start routine */
										(LPVOID)sAccept,			/* parameter (socket) */
										0,					/* creation flags (start immediately) */
										&dwThreadID);		/* thread id */
				break;
			case MPI_SERVE_CREATE_SINGLE_PROCESS:
				WriteDebugLog("It is a request to start a single process");
				hThread = CreateThread(NULL,
										0,
										MPIServeCreateSingleProcess,
										(LPVOID)sAccept,
										0,
										&dwThreadID);
				break;
			case MPI_SERVE_KILL_SINGLE_PROCESS:
				WriteDebugLog("It is a request to kill a single process");
				hThread = CreateThread(NULL,
										0,
										MPIServeKillSingleProcess,
										(LPVOID)sAccept,
										0,
										&dwThreadID);
				break;
			case MPI_SERVE_START_MPI_JOB:
				WriteDebugLog("It is a request to start an MPI Job");
				hThread = CreateThread(NULL,
										0,
										MPIServeStartMPIJob,
										(LPVOID)sAccept,
										0,
										&dwThreadID);
				break;
			case MPI_KILL_MPI_JOB:
				WriteDebugLog("It is a request to kill an MPI Job");
				hThread = CreateThread(NULL,
										0,
										MPIServeKillMPIJob,
										(LPVOID)sAccept,
										0,
										&dwThreadID);
				break;
			default:
				break;
			}
		}
	}
cleanup:
	if (ERROR_SUCCESS != nStatus)
	{
		TCHAR szMsg[512];
		GetLastErrorText(szMsg,512);
		AddToMessageLog(szMsg);
	}
	if (INVALID_SOCKET != sListener)
		closesocket(sListener);
	if (INVALID_SOCKET != sAccept)
		closesocket(sAccept);
	if (wsaInit)
		WSACleanup();
	return nStatus;
}
